""" Make sure our fetcher factory functions fine, friend """

from typing import NamedTuple

# pylint: disable=E0239, E0401, E0402, R0201, R0903

from productfetcher.fetcher_factory import ConfiguredFetcherFactory
from productfetcher.fetchers import (
    DryRunFakeFileFetcher,
    NgasStreamingFileFetcher,
    NgasDirectCopyFileFetcher,
    NgasModeDetectingFetcher,
)


class FakeFile(NamedTuple):
    """Ceci n’est pas une file"""

    size: int
    checksum: int

    # direct copy uses this
    def can_direct_copy(self) -> bool:
        return True


def test_configured_fetcher_factory():
    """
    Various tests of the fetcher factory

    :return:
    """
    fake_file = FakeFile(239823983, -329831113)

    # let's try a few combinations and make sure we find reasonable things
    # first, let's make sure the concurrency value is used properly
    factory = ConfiguredFetcherFactory(concurrency=732)
    assert factory.fetch_plan([]).concurrency == 732

    # let's also make sure that if we specify a lower concurrency value, it gets used instead
    assert factory.fetch_plan([], concurrency=3).concurrency == 3

    # now let's make sure that if we ask for a dry run, that's what we get
    factory = ConfiguredFetcherFactory(dry_run=True)
    assert isinstance(factory.fetch_ngas(fake_file).underlying.underlying, DryRunFakeFileFetcher)
    assert isinstance(
        factory.fetch_oracle_xml(fake_file).underlying.underlying, DryRunFakeFileFetcher
    )

    # now let's make sure that if we ask for streaming or direct copy, we get those
    factory = ConfiguredFetcherFactory(streaming=True)
    assert isinstance(factory.fetch_ngas(fake_file).underlying.underlying, NgasStreamingFileFetcher)
    factory = ConfiguredFetcherFactory(direct_copy=True)
    assert isinstance(
        factory.fetch_ngas(fake_file).underlying.underlying, NgasDirectCopyFileFetcher
    )

    # and let's make sure that if we don't specify streaming or direct copy, we get a detection
    factory = ConfiguredFetcherFactory(streaming=False, direct_copy=False)
    assert isinstance(factory.fetch_ngas(fake_file).underlying.underlying, NgasModeDetectingFetcher)
