"""
Tests for System Mediator package arbitrator
"""
import logging
import sys
from typing import List
from unittest.mock import patch, MagicMock

import pytest
import argparse

from testing.utils.conftest import mock_workflow_info, mock_capability_info

from system_mediator.arbitrator import Arbiter
from workspaces.capability.schema import CapabilityExecution, CapabilityRequest

from workspaces.capability.services.capability_info import CapabilityInfo
from workspaces.workflow.schema import WorkflowRequest
from workspaces.workflow.services.workflow_info import WorkflowInfo

pytest_plugins = ["testing.utils.conftest"]

logger = logging.Logger("test_arbitrator")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))

ns = argparse.Namespace(force_fail=['-1'])


@patch("system_mediator.arbitrator.Router", MagicMock())
@pytest.mark.usefixtures("mock_workflow_info", "mock_capability_info")
def mock_arbiter(mock_workflow_info: WorkflowInfo, mock_capability_info: CapabilityInfo) -> Arbiter:
    return Arbiter(mock_workflow_info, mock_capability_info, ns)


@pytest.mark.usefixtures("mock_workflow_requests")
def mock_message(mock_workflow_requests: List[WorkflowRequest]):
    failed_msg = {
        "service": "workflow",
        "routing_key": "workflow",
        "subject": mock_workflow_requests[0].__json__(),
        "type": "workflow-failed"
    }
    return failed_msg


arbiter = mock_arbiter(mock_workflow_info=mock_workflow_info, mock_capability_info=mock_capability_info)


@pytest.mark.usefixtures("mock_workflow_info",
                         "mock_capability_info",
                         "mock_workflow_requests",
                         "mock_capability_execution",
                         "mock_capability_requests")
class TestArbiter:

    def test_which_option(self):
        arbiter.which_option(ns)
        assert arbiter.intervention == "force_fail"
        assert arbiter.wf_request_id == '-1'

    @pytest.mark.skip("ignores router mock?")
    @pytest.mark.usefixtures("mock_workflow_requests")
    def test_intervene_fail(self, mock_workflow_info: WorkflowInfo, mock_workflow_requests: List[WorkflowRequest]):
        with patch("messaging.router.Router") as mock_router:
            mock_workflow_info.lookup_workflow_definition = MagicMock(return_value=mock_workflow_requests[0])
            arbiter.info = mock_workflow_info
            arbiter.message_router = mock_router
            arbiter.intervene_fail(arbiter.wf_request_id)
            assert arbiter.message_router.send_message.call_count == 1

    @pytest.mark.usefixtures("mock_workflow_requests", "mock_capability_execution")
    def test_resolve_fail(self,
                          mock_workflow_requests: List[WorkflowRequest],
                          mock_capability_execution: CapabilityExecution):
        arbiter.info.lookup_workflow_request = MagicMock(return_value=mock_workflow_requests[0])
        arbiter.cinfo.lookup_execution_by_workflow_request_id = MagicMock(return_value=mock_capability_execution)
        assert arbiter.resolve_fail(arbiter.wf_request_id) is None

    @pytest.mark.usefixtures("mock_workflow_requests", "mock_capability_execution")
    def test_check_status(self,
                          mock_capability_info: CapabilityInfo,
                          mock_workflow_requests: List[WorkflowRequest],
                          mock_capability_execution: CapabilityExecution):
        arbiter.cinfo = mock_capability_info
        arbiter.info.lookup_workflow_request = MagicMock(return_value=mock_workflow_requests[0])
        arbiter.cinfo.lookup_execution_by_workflow_request_id = MagicMock(return_value=mock_capability_execution)

        assert arbiter.check_status(arbiter.wf_request_id) is True

    def test_intercede(self):
        assert arbiter.intercede() is None
