import { Component, OnInit } from "@angular/core";
import { CapabilityRequest } from "../../model/capability-request";
import { ActivatedRoute } from "@angular/router";
import { DataRetrieverService } from "../../services/data-retriever.service";
import { CapabilityExecution } from "../../model/capability-execution";
import { Capability } from "../../model/capability";

@Component({
  selector: "app-capability-request",
  templateUrl: "./capability-request.component.html",
  styleUrls: ["./capability-request.component.scss"],
})
export class CapabilityRequestComponent implements OnInit {
  public capabilityRequest: CapabilityRequest;
  public capability: Capability;
  public capabilityExecution: CapabilityExecution;

  private capabilityRequestObserver = {
    next: (request) => {
      this.capabilityRequest = request;
      this.capabilityExecution = request.current_execution;
      this.dataRetriever
        .getCapability(this.capabilityRequest.capability_name)
        .subscribe(this.capabilityObserver);
    },
    error: (error) => console.error("Error when retrieving capability request:" + error),
  };
  private capabilityObserver = {
    next: (capability) => {
      this.capability = capability;
    },
    error: (error) => console.error("Error when retrieving capability:" + error),
  };

  constructor(private route: ActivatedRoute, private dataRetriever: DataRetrieverService) {
    const requestID = parseInt(this.route.snapshot.paramMap.get("id"));
    this.dataRetriever.getCapabilityRequest(requestID).subscribe(this.capabilityRequestObserver);
  }

  ngOnInit(): void {}
}
