"""
Tests for the functionality of the Notification REST API.
The logic can be found in `notification/views/notify/py`.
"""
from unittest.mock import patch

import pytest
from pyramid.testing import DummyRequest

import notification.views.notify
from workspaces.notification.services.notification_service import NotificationService


@pytest.mark.usefixtures("dummy_send", "mock_notification_service")
class TestNotifyViews:

    def test_view_notifications(self, dummy_send: DummyRequest):
        from notification.views.notify import list_notification_templates

        expected_response = "Available notification templates:\nemail\n"
        response = list_notification_templates(dummy_send)
        assert response.status_code == 200
        assert response.text == expected_response

    def test_send_notification(self, dummy_send: DummyRequest, mock_notification_service: NotificationService):
        from notification.views.notify import send_notification

        dummy_send.json_body = {
            "destination_email": "chausman@nrao.edu",
            "subject": "first thing",
            "message": "second thing"
        }

        dummy_send.matchdict["template_name"] = "email"
        with patch("workspaces.notification.services.notification_service.NotificationService.send_email"):
            response = send_notification(dummy_send)
        assert response.status_code == 200

    def test_create_notification(self, dummy_send: DummyRequest):
        from notification.views.notify import create_notification_template

        expected_response = "Template new_template created."
        dummy_send.json_body = {"name": "new_template",
                                "description": "a test template",
                                "template": "Subject: {{foo}}\n\n{{bar}}"}

        response = create_notification_template(dummy_send)
        assert response.status_code == 200
        assert response.text == expected_response

    def test_delete_notification(self, dummy_send: DummyRequest):
        from notification.views.notify import delete_notification_template

        expected_response = "Template new_template deleted."
        dummy_send.matchdict["template_name"] = "new_template"

        response = delete_notification_template(dummy_send)
        assert response.status_code == 200
        assert response.text == expected_response
